import {
    BlockLocation, Location, Vector,
    ItemStack, ItemTypes,
    EntityTypes, 
    MinecraftItemTypes, 
    MinecraftEffectTypes,
    MinecraftDimensionTypes,
    world, system
} from '@minecraft/server';
let players = [];

const mobs = [
    "allay",
    "armor_stand",
    "axolotl",
    "bat",
    "bee",
    "blaze",
    "boat",
    "cat",
    "cave_spider",
    "chicken",
    "cod",
    "cow",
    "creeper",
    "dolphin",
    "donkey",
    "drowned",
    "elder_guardian",
    "ender_dragon",
    "enderman",
    "endermite",
    "evocation_illager",
    "fox",
    "frog",
    "ghast",
    "glow_squid",
    "guardian",
    "hoglin",
    "horse",
    "husk",
    "iron_golem",
    "llama",
    "magma_cube",
    "mooshroom",
    "mule",
    "npc",
    "ocelot",
    "panda",
    "parrot",
    "phantom",
    "pig",
    "piglin",
    "piglin_brute",
    "pillager",
    "player",
    "polar_bear",
    "pufferfish",
    "rabbit",
    "ravager",
    "salmon",
    "sheep",
    "shulker",
    "silverfish",
    "skeleton",
    "skeleton_horse",
    "slime",
    "snow_golem",
    "spider",
    "squid",
    "stray",
    "strider",
    "tadpole",
    "tnt",
    "trader_llama",
    "tropicalfish",
    "turtle",
    "vex",
    "villager",
    "villager_v2",
    "vindicator",
    "wandering_trader",
    "warden",
    "witch",
    "wither",
    "wither_skeleton",
    "wolf",
    "zoglin",
    "zombie",
    "zombie_horse",
    "zombie_pigman",
    "zombie_villager_v2"
], effects = [
    'absorption',       // --
    'conduit_power',    //  |
    'fire_resistance',  //  |  A
    'haste',            //  |
    'health_boost',     //  |
    'invisibility',     // --

    'jump_boost',       // --
    'night_vision',     //  |
    'regeneration',     //  |  B
    'resistance',       //  |
    'slow_falling',     //  |
    'speed',            // --

    'strength',         // --
    'village_hero',     //  |
    'water_breathing',  //  |  C
    'bad_omen',         //  |
    'blindness',        //  |
    'darkness',         // --

    'hunger',           // --
    'levitation',       //  |
    'mining_fatigue',   //  |  D
    'nausea',           //  |
    'poison',           //  |
    'slowness',         // --

    'weakness',         // --  E
    'wither'            // --
];

world.events.playerJoin.subscribe(e => {
    let player = e.player;
    player.currentDir = -1;
    player.currentId = -1;
    player.pointer = null;
    player.active = false;
    player.triggerEvent(`ssg:name_0`);
    player.triggerEvent(`ssg:type_0`);
    player.triggerEvent(`ssg:health_0`);
    player.triggerEvent(`ssg:max_health_0`);
    for (let a of 'abcde') player.triggerEvent(`ssg:effect_${a}_000000`);
    players[player.name] = player;
});
world.events.playerLeave.subscribe(e => {
    for (let p in players) {
        if (e.playerName == p) {
            if (players[p].pointer) players[p].pointer.triggerEvent('ssg:despawn');
            delete players[p];
        }
    }
});

world.events.itemStartCharge.subscribe(e => {
    if (!e.itemStack.typeId.endsWith('spyglass') || !e.source.typeId.endsWith('player')) return;
    e.source.triggerEvent('ssg:activate_1');
    e.source.active = true;
});
world.events.itemReleaseCharge.subscribe(e => {
    if (!e.itemStack.typeId.endsWith('spyglass') || !e.source.typeId.endsWith('player')) return;
    e.source.triggerEvent('ssg:activate_0');
    e.source.active = false;
});

world.events.tick.subscribe(_ => {
    try {
        for (let i in players) {
            let p = players[i], rot = players[i].rotation.y;
            if (rot > -22.5 && rot < 22.5 && p.currentDir != 0) p.currentDir = 0;
            else if (rot > 22.5 && rot < 67.5 && p.currentDir != 1) p.currentDir = 1;
            else if (rot > 67.5 && rot < 112.5 && p.currentDir != 2) p.currentDir = 2;
            else if (rot > 112.5 && rot < 157.5 && p.currentDir != 3) p.currentDir = 3;
            else if ((rot > 157.5 || (rot > -180 && rot < -157.5)) && p.currentDir != 4) p.currentDir = 4;
            else if (rot > -157.5 && rot < -112.5 && p.currentDir != 5) p.currentDir = 5;
            else if (rot > -112.5 && rot < -67.5 && p.currentDir != 6) p.currentDir = 6;
            else if (rot > -67.5 && rot < -22.5 && p.currentDir != 7) p.currentDir = 7;
            p.triggerEvent(`ssg:dir_${p.currentDir}`);
            let e = p.getEntitiesFromViewVector()[0], n = -1;
            let health = 0, max_health = 0;
            if (e && p.active) {
                if (p.currentId != e.id) removePointer(p);
                p.currentId = e.id;
                spawnPointer(p, e.location);

                n = mobs.indexOf(e.typeId.split(':')[1]);
                let hCom = e.getComponent('health');
                if (hCom) health = hCom.current;
                if (hCom) max_health = hCom.value;
                if (p.pointer) p.pointer.teleport(e.location, p.dimension, 0, 0, false);

                let efs = new Array(effects.length).fill(0);
                for (let et in MinecraftEffectTypes) {
                    if (!e.getEffect(MinecraftEffectTypes[et])) continue;
                    for (let ef in effects) {
                        if (MinecraftEffectTypes[et].getName().endsWith('poison')) efs[22] = 1;
                        else if (MinecraftEffectTypes[et].getName() == effects[ef]) efs[ef] = 1;
                    }
                }
                p.triggerEvent(`ssg:effect_a_${efs[0]}${efs[1]}${efs[2]}${efs[3]}${efs[4]}${efs[5]}`);
                p.triggerEvent(`ssg:effect_b_${efs[6]}${efs[7]}${efs[8]}${efs[9]}${efs[10]}${efs[11]}`);
                p.triggerEvent(`ssg:effect_c_${efs[12]}${efs[13]}${efs[14]}${efs[15]}${efs[16]}${efs[17]}`);
                p.triggerEvent(`ssg:effect_d_${efs[18]}${efs[19]}${efs[20]}${efs[21]}${efs[22]}${efs[23]}`);
                p.triggerEvent(`ssg:effect_e_${efs[24]}${efs[25]}0000`);

            } else removePointer(p);
            p.triggerEvent(`ssg:name_${n}`);
            p.triggerEvent(`ssg:health_${health}`);
            p.triggerEvent(`ssg:max_health_${max_health}`);
        }
    } catch (err) { }
});
system.events.beforeWatchdogTerminate.subscribe(e => e.cancel = true);

function spawnPointer(p, location) {
    try {
        if (p.pointer != null) return;
        let loc = new Location(location.x, location.y, location.z), id = Math.abs(p.id) + '', twos = [], as = 'abcdef';
        p.pointer = p.dimension.spawnEntity(`ssg:test`, loc);
        p.pointer.addTag(p.id+'');
        while (id.length > 0) {
            twos.push(id.substring(id.length - 2));
            id = id.substring(0, id.length - 2)
        }
        for (let i in twos) {
            p.pointer.triggerEvent(`ssg:${as[i]}_${twos[i]}`);
            p.triggerEvent(`ssg:${as[i]}_${twos[i]}`);
        }
    } catch (err) { throw err; }
}

function removePointer(p) {
    try {
        if (p.pointer == null) return;
        p.pointer.triggerEvent('ssg:despawn');
        p.pointer = null;
        p.currentId = -1;
    } catch (err) { throw err; }
}
//p.onScreenDisplay.setActionBar();